import json
import logging

from redis import Redis

from settings import redis_cache
from .util import Singleton

logger = logging.getLogger()

CHUNK_SIZE = 5000


class BaseRedisConnector(Redis):
    def __init__(self):
        try:
            super().__init__(host=redis_cache['redis_host'], port=redis_cache['redis_port'],
                             password=redis_cache['redis_pass'])

        except Exception as error:
            logger.error(error)
            raise error


class RedisConnector(BaseRedisConnector, metaclass=Singleton):
    pass


class RedisCache(RedisConnector):
    def __init__(self):
        super().__init__()

    def get_cache_multiple_value(self, key, custom_value_name):
        if key:
            __json_value_data = self.get(key)
            if __json_value_data is not None:
                __value = (json.loads(__json_value_data)).get(custom_value_name, None)
                return __value
        return None

    def set_cache_multiple_value(self, key, value, custom_value_name, ttl=60):
        __json_value_data = self.get(key)
        try:
            __exist_json = json.loads(__json_value_data)
            __dict_value = {custom_value_name: value}
            __dict_value.update(__exist_json)
            __json_data = json.dumps(__dict_value)
            __cache_status = self.set(key, __json_data, ex=ttl)
            return __cache_status
        except Exception as error:
            logger.error(error)
            __json_data = json.dumps({custom_value_name: str(value)})
            __cache_status = self.set(key, __json_data, ex=ttl)
            return __cache_status

    def clear_ns(self, ns):
        cursor = '0'
        ns_keys = ns + '*'
        while cursor != 0:
            cursor, keys = self.scan(cursor=cursor, match=ns_keys, count=CHUNK_SIZE)
            if keys:
                self.delete(*keys)
        return True



import os
import sys
import pathlib
import logging
from abc import ABC, abstractmethod

import rootpath
import sentry_sdk
from shutil import copy
from pydoc import locate
from getpass import getpass
from alembic import command
from copy import deepcopy
from alembic.config import Config
from sqlalchemy_searchable import sync_trigger
from alembic.command import init as alembic_init
from .api_cache import CachePlugin
from .db_initializer import DataBaseConnectionManager
from .filters import SmartFiltersPlugin
from .plugin import DBInjectorPlugin
from .util import strip_path
from .wrappers import db_handler, cors
from ICECREAM.baseapp import BaseApp, PluginApp
from .models.query import get_or_create
from bottle import Bottle, run, static_file, BaseTemplate
from sentry_sdk.integrations.bottle import BottleIntegration
from sentry_sdk.integrations.logging import LoggingIntegration

try:
    from settings import default_address, sentry_dsn, DEBUG, media_path, redis_cache, database
except ModuleNotFoundError as exception:
    from ICECREAM.settings import default_address, sentry_dsn, DEBUG, media_path, redis_cache

    logging.error(exception)


def get_default_address():
    _default = default_address
    return _default


rootpath.append()
root_name, _, _ = __name__.partition('.')
root_module = sys.modules[root_name]
ICECREAM_PATH = os.path.dirname(root_module.__file__)

list_files = ['models.py', 'controller.py', 'schemas.py', 'urls.py']


class CommandsParser(object):
    def __init__(self, opt_commands):
        self.command = []
        self.subcommands = []
        self.opt_commands = opt_commands
        self.commands_list = ['startapp', 'runserver', 'wsgi', 'makemigrations', 'createsuperuser', 'index_search']

    def get_command(self, ):
        if self.opt_commands[0] in self.commands_list:
            self.command = self.opt_commands[0]
            return self.command

    def get_subcommand(self, ):
        if self.opt_commands[0] in self.commands_list:
            self.command = self.opt_commands[0]
            self.subcommands = deepcopy(self.opt_commands)
            self.subcommands.remove(self.command)
        return self.subcommands

    def has_value(self) -> bool:
        if not self.opt_commands:
            return True
        return False

    def has_subcommand(self) -> bool:
        if not self.get_subcommand():
            return False
        return True


class CommandManager(object):

    def __init__(self, opt_commands):
        self.command = CommandsParser(opt_commands)

    def execute(self):
        if self.command.has_value():
            core = Core(db_type=database["db_type"])
            return core.execute_wsgi()
        if self.command.get_command() == 'makemigrations':
            if self.command.has_subcommand():
                self.makemigrations(self.command.get_subcommand()[0])
            else:
                self.makemigrations("migration")
        if self.command.get_command() == "index_search":
            self.index_search()
        if self.command.get_command() == "createsuperuser":
            self.create_super_user()
        if self.command.get_command() == 'startapp':
            if self.command.has_subcommand():
                self.create_app(self.command.get_subcommand()[0])
            else:
                sys.stdout.write('ICECREAM: Need to provide an app name' + '\n')
        elif self.command.get_command() == 'runserver':
            core = Core(db_type=database["db_type"])
            if self.command.has_subcommand():
                return core.execute_runserver(self.command.get_subcommand()[0])
            else:
                return core.execute_runserver(None)

    @staticmethod
    def create_app(app_name):
        try:
            path = app_name
            if not os.path.exists(path):
                os.makedirs(path)
            for file in list_files:
                filename = file
                with open(os.path.join(path, filename), 'wb') as temp_file:
                    temp_file.write('"ICECREAM"'.encode())
        except IOError as err:
            raise err.filename

    @staticmethod
    def makemigrations(commit_name):
        project_root = os.path.abspath(os.curdir)
        alembic_folder_dst = project_root + os.sep + "alembic"
        alembic_init_dst = project_root + os.sep + "alembic.ini"
        alembic_cfg = Config()
        alembic_cfg.config_file_name = alembic_init_dst
        if not (os.path.exists(alembic_folder_dst) and os.path.exists(alembic_init_dst)):
            alembic_init(directory=alembic_folder_dst, config=alembic_cfg)
            src = ICECREAM_PATH + "/migration_tool/env.py"
            copy(src, alembic_folder_dst)
        command.revision(alembic_cfg, commit_name, autogenerate=True)

    @db_handler
    def create_super_user(self, db_session):
        try:
            from app_user.schemas import user_serializer, superuser_serializer
            from app_user.models import User, Person
            name = input("Name:")
            person = get_or_create(Person, db_session, Person.name == name)
            person.last_name = input("LastName:")
            password = getpass("Password: ")
            phone = input("Phone:")
            person.name = name
            person.email = input("Email:")
            db_session.add(person)
            user = get_or_create(User, db_session, User.phone == phone)
            user.username = input("UserName:")
            user.phone = phone
            user.set_roles(["admin"])
            user.set_password(password)
            user.person = person
            # superuser_serializer.load(user_serializer.dump(user))
            db_session.add(user)
            db_session.commit()
            result = user_serializer.dump(db_session.query(User).get(user.id))
            return result
        except Exception as e:
            logging.info(e.args)

    @staticmethod
    def index_search():
        try:
            from settings import searches_index
            for _index in searches_index:
                db = DataBaseConnectionManager().db
                sync_trigger(db.engine, *_index)
        except Exception:
            raise


class CoreAbstract(ABC):
    def __init__(self):
        self.core = Bottle()


class Core(CoreAbstract):
    def __init__(self, **options):
        try:
            super().__init__()
            self.options = options
            BaseTemplate.defaults['get_url'] = self.core.get_url
            self.__route_homepage()
            self.__init_jwt()
            self.__install_plugins()
            self.__route_file_server()
            self.__init_api_cache()
            self.__initialize_log()
            self.__initialize_filters()
            self.__init_cors()
            self.__init_inject_db(self.options)
            self.__initialize_sentry_log()
            self.__register_routers()
            self.__route_404_error()
            # self.__initialize_throttle()
            logging.info("ICECREAM initialized")

        except Exception as e:
            logging.error("icecream core cannot initialize")
            raise ValueError(e)

    def __route_404_error(self):
        @self.core.error(404)
        def custom_page(error):
            return "ICECREAM didn't find anything"

    def __init_jwt(self):
        try:
            from app_user.authentication import jwt_plugin
            if jwt_plugin.auth_endpoint:
                self.core.install(jwt_plugin)
                return True
            return False
        except ModuleNotFoundError as ex:
            logging.info(ex)

    def __init_cors(self):
        self.core.install(cors)

    def __init_inject_db(self, options=None):
        if options.get('db_type'):
            _db_plugin = DBInjectorPlugin(db_type=options.get('db_type'))
        else:
            _db_plugin = DBInjectorPlugin(db_type="sqlite")
        return self.core.install(_db_plugin)

    def __init_api_cache(self):
        cache = CachePlugin('url_cache', 'redis', host=redis_cache['redis_host'], port=redis_cache['redis_port'],
                            password=redis_cache['redis_pass'])
        self.core.install(cache)

    def __route_homepage(self, ):
        if DEBUG:
            self.core.hook('before_request')(strip_path)
            self.core.route('/api', callback=self._serve_homepage_template)
            self.core.route('/api/icecream/static/<filepath:path>', callback=self.server_homepage_static)

    def __route_file_server(self):
        self.core.hook('before_request')(strip_path)
        self.core.route('/api/media/<filepath:path>', callback=self.__serve_static_media)

    @staticmethod
    def _serve_homepage_template():
        __homepage_file = static_file("index.html",
                                      root=ICECREAM_PATH + '/statics/templates')
        return __homepage_file

    @staticmethod
    def server_homepage_static(filepath):
        return static_file(filepath, root=ICECREAM_PATH + '/statics/images/')

    @staticmethod
    def __serve_static_media(filepath):
        return static_file(filepath, root=media_path)

    def execute_wsgi(self):
        try:
            return self.core
        except Exception:
            raise

    def execute_runserver(self, address=None):
        try:
            __address = self.__convert_command_to_address(address)
            run(self.core, host=__address['host'], port=__address['port'], server='tornado', debug=DEBUG, reloader=True)
            return self.core
        except Exception as err:
            sys.stdout.write('execute runserver has problem')
            raise err

    @staticmethod
    def __convert_command_to_address(argv):
        try:
            _address = get_default_address()
            if argv is not None:
                arg_address = argv.split(':')
                _address['host'] = arg_address[0]
                _address['port'] = arg_address[1]
        except Exception as e:
            raise ValueError('ICECREAM: Please provide a valid address')
        return _address

    @staticmethod
    def __initialize_log():
        log_formatter = logging.Formatter("%(asctime)s [%(threadName)-12.12s] [%(levelname)-5.5s]  %(message)s")
        root_logger = logging.getLogger()

        console_handler = logging.StreamHandler()
        console_handler.setFormatter(log_formatter)
        root_logger.addHandler(console_handler)

        file_handler = logging.FileHandler(filename='icecream.log', mode='a')
        file_handler.setLevel(level=logging.ERROR)
        file_handler.setFormatter(log_formatter)
        root_logger.addHandler(file_handler)
        root_logger.setLevel(level=logging.INFO)

    @staticmethod
    def __initialize_sentry_log():
        sentry_logging = LoggingIntegration(
            level=logging.INFO,  # Capture info and above as breadcrumbs
            event_level=logging.ERROR  # Send errors as events
        )
        sentry_sdk.init(
            dsn=sentry_dsn,
            integrations=[BottleIntegration(), sentry_logging]
        )

    @staticmethod
    def __initialize_baseapps():
        try:
            from settings import apps
            for app in apps:
                baseapp_class = locate(app)
                if baseapp_class:
                    instance = baseapp_class()
        except Exception as exception:
            logging.error(exception)
            raise ValueError("undefined app")

    def __initialize_plugins(self):
        try:
            from settings import plugins
            for plugin in plugins:
                plugin_class = locate(plugin)
                if plugin_class:
                    print(plugin_class)
                    # instance = plugin_class()
        except Exception as exception:
            logging.error(exception)
            raise ValueError("undefined app")

    def __get_subclasses(self, cls):
        for subclass in cls.__subclasses__():
            yield from self.__get_subclasses(subclass)
            yield subclass

    def __register_routers(self):
        self.__initialize_baseapps()
        base_app_subclasses = self.__get_subclasses(BaseApp)
        for sub_class in base_app_subclasses:
            sub_class.call_router(sub_class, core=self.core)

    def __install_plugins(self):
        self.__initialize_plugins()
        base_app_subclasses = self.__get_subclasses(PluginApp)
        for sub_class in base_app_subclasses:
            sub_class.install(sub_class, core=self.core)

    def __initialize_filters(self):
        self.core.install(SmartFiltersPlugin())



import os
import secrets
from distutils.util import strtobool
from bottle import request


class Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]


def generate_otp_code():
    return secrets.SystemRandom().randrange(999, 9999)


def strip_path():
    request.environ['PATH_INFO'] = request.environ['PATH_INFO'].rstrip('/')


def get_media_link(file_name):
    host = request.get_header('host')
    # for production
    return 'http://{}/api/media/{}'.format(host, file_name)


def str_to_bool(val):
    try:
        return strtobool(val)
    except Exception as e:
        return None


def detect_file_path(task_file="tasks.py", exclude_folders=None):
    if exclude_folders is None:
        exclude_folders = []
    project_root = os.getcwd()
    tasks = []
    file_path = os.path.join(project_root)
    for root, dirs, files in os.walk(file_path):
        files = [filename for filename in files if not filename[0] == '.']
        dirs[:] = [dirname for dirname in dirs if not dirname[0] == '.']
        dirs[:] = [dirname for dirname in dirs if dirname not in exclude_folders]
        for filename in files:
            if root is not project_root:
                if os.path.basename(root) == 'tasks' or filename == task_file:
                    dirname = root.split(os.path.sep)[-1]
                    if filename != '__init__.py' and filename.endswith('.py'):
                        task = os.path.join(dirname, filename).replace('/', '.').replace('.py', '')
                        tasks.append(task)
    return tuple(tasks)



import logging
from sqlalchemy import orm
from bottle import request

from ICECREAM.util import Singleton
from settings import database
from mongosql import MongoSqlBase
from sqlalchemy.orm import Session
from sqla_wrapper import SQLAlchemy
from abc import ABCMeta, abstractmethod
from sqlalchemy_searchable import make_searchable
from sqlalchemy.ext.declarative import declarative_base

logger = logging.getLogger()
Base = declarative_base(cls=(MongoSqlBase,))
metadata = Base.metadata


class ConnectionFactory(object):
    __metaclass__ = ABCMeta

    def connect(self):
        return SQLAlchemy(self.get_database_uri(), pool_pre_ping=True)

    @abstractmethod
    def get_database_uri(self):
        raise NotImplementedError()


class PostgresConnectionFactory(ConnectionFactory):
    def __init__(self, database_conf: {}, meta_data=None):
        super().__init__()
        self.user = database_conf['db_user']
        self.password = database_conf['db_pass']
        self.host = database_conf['db_host']
        self.database = database_conf['db_name']
        self.port = database_conf['db_port']
        make_searchable(meta_data)

    def get_database_uri(self):
        uri = 'postgresql+psycopg2://{}:{}@{}:{}/{}'.format(self.user, self.password,
                                                            self.host, self.port,
                                                            self.database)
        return uri


class Sqlite3ConnectionFactory(ConnectionFactory):
    def __init__(self, db_name="db"):
        self.__db = db_name

    def get_database_uri(self):
        return "sqlite:///{}.sqlite3".format(self.__db)


class SqliteTestConnectionFactory(Sqlite3ConnectionFactory):

    def get_database_uri(self):
        return 'sqlite:///:memory:'


class BaseDataBaseConnectionManager(object):
    def __init__(self, db_type=database["db_type"]):
        self.db = self.create_db_connection(database, metadata, db_type=db_type)
        self.db.session_options = {'autocommit': True}
        metadata.create_all(self.db.engine)
        orm.configure_mappers()

    def get_db_session(self) -> Session:
        if hasattr(request, 'db_session'):
            return getattr(request, 'db_session')
        else:
            setattr(request, 'db_session', self.db.session)
            return getattr(request, 'db_session')

    @staticmethod
    def create_db_connection(database_conf, meta_data, db_type="sqlite"):
        logging.info("DB is connecting ...")
        if db_type == "sqlite":
            logging.info("sqlite")
            return Sqlite3ConnectionFactory().connect()
        elif db_type == "postgres":
            logging.info("postgres")
            return PostgresConnectionFactory(database_conf, meta_data).connect()
        elif db_type == "memory":
            logging.info("sqlite memory")
            return SqliteTestConnectionFactory().connect()
        else:
            raise Exception(f"Not implemented {db_type}")


class DataBaseConnectionManager(BaseDataBaseConnectionManager, metaclass=Singleton):
    pass



def make_project():
    import os
    import sys
    project_root = os.path.abspath(os.curdir)
    root_name, _, _ = __name__.partition('.')
    root_module = sys.modules[root_name]
    ice_cream_root = os.path.dirname(root_module.__file__)
    assets_address = ice_cream_root + os.sep + 'assets' + os.sep + 'project_assets.zip'

    import zipfile
    with zipfile.ZipFile(assets_address, 'r') as zip_ref:
        zip_ref.extractall(project_root)



import json

from bottle import HTTPResponse as BottleHttpResponse


class HTTPResponse(BottleHttpResponse):
    default_status = 500

    def __init__(self, status=None, body=None, headers=None, exception=None, message="msg", traceback=None,
                 **options):
        self.exception = exception
        self.traceback = traceback
        self.body = self.convert_to_json(body, message)
        headers = headers or {'Content-type': 'application/json'}
        super(HTTPResponse, self).__init__(self.body, headers=headers, status=status, **options)

    @staticmethod
    def convert_to_json(body, message):
        if body is None:
            return body
        if type(body) is dict:
            return json.dumps(body)
        return json.dumps({message: body})


class HTTPError(HTTPResponse):
    pass



# -*- coding: utf-8 -*-
"""`bottle_cache.backend` module.

Provides package caching backend implementations.
"""

__author__ = 'Papavassiliou Vassilis'
__date__ = '23-1-2016'

from redis import StrictRedis
from redis.exceptions import RedisError

from ICECREAM.api_cache.bases import BaseCacheBackend, CacheError


class RedisCacheBackend(BaseCacheBackend):
    """Redis backend Implementation
    """

    def __init__(self, backend_client=StrictRedis, key_tpl='{}', **conn_data):
        """Overriding 'BaseCache.__init__' method.
        """

        redis_backend = backend_client(**conn_data)

        super(RedisCacheBackend, self).__init__(backend_client=redis_backend, key_tpl=key_tpl)

    def get(self, key):
        """Implementing `BaseCache.get` method.
        """
        try:
            return self.backend.get(key)
        except RedisError as error:  # pragma: no cover
            raise CacheError(error.args)

    def set(self, key, value, ttl=None):
        """Implementing `BaseCache.set` method.
        """
        try:
            if ttl:
                self.backend.setex(key, ttl, value)
            else:
                self.backend.set(key, value)

            return self

        except RedisError as error:  # pragma: no cover
            raise CacheError(error.args)

    def remove(self, key):
        """Implementing `BaseCache.remove` method.
        """
        try:
            self.backend.delete(key)
            return self

        except RedisError as error:  # pragma: no cover
            raise CacheError(error.args)

    def clear(self):
        self.backend.flushall()
        return self



# -*- coding: utf-8 -*-
"""`bottle_cache.plugin` module.

Provides bottle.py cache plugin.
modified by amir lesani
"""

__author__ = 'Papavassiliou Vassilis'
__date__ = '23-1-2016'

import bottle
import collections
import inspect
import ujson

from ICECREAM.api_cache import backend

CacheInfo = collections.namedtuple('CacheInfo', 'ttl, cache_key_func, content_type')


def available_backend():  # pragma: no cover
    """Auto loading available cache backend implementation
    without hard-coding them.
    """

    backend_module = backend

    available_list = [
        callback for callback in dir(backend_module)
        if 'cachebackend' in callback.lower() and
           'base' not in callback.lower()
    ]

    return {callback.lower().replace('cachebackend', ''): getattr(backend_module, callback)
            for callback in available_list}


def cache_for(ttl, cache_key_func='full_path', content_type='application/json'):
    """A decorator that signs a callable object with a 'CacheInfo'
    attribute.

    Args:
        ttl (int): Cache Time to live in seconds.
        cache_key_func (str): The key for the cache key function installed on Plugin.
        content_type (str): Handler response type.
    Returns:
        The callable object.
    """

    def _wrapped(callback):
        setattr(
            callback,
            'cache_info',
            CacheInfo(ttl, cache_key_func, content_type)
        )

        return callback

    return _wrapped


class CachePlugin(object):
    """A `bottle.Bottle` application plugin for `bottle_cache.backend` implementations.

    Attributes:
        keyword (str): The string keyword for application registry.
        provider (instance): A JWTProvider instance.
        login_enable (bool): If True app is mounted with a login handler.
        auth_endpoint (str): The authentication uri for provider if
                             login_enabled is True.
        kwargs : JWTProvider init parameters.
    """

    api = 2

    content_types = (
        ('text/html', lambda x: x),
        ('application/json', ujson.dumps)
    )

    cache_key_rules = {
        'full_path': lambda req, cxt: str(cxt.rule) + req.query_string,
        'query_path': lambda req, cxt: req.query_string,
    }

    def __init__(self, keyword, backend, **backend_kwargs):  # pragma: no cover

        if backend not in available_backend():
            raise bottle.PluginError(
                'Invalid backend {} provided. Available until now: ({})'.format(
                    backend, ', '.join(available_backend().keys())
                )
            )

        self.backend = available_backend()[backend](**backend_kwargs)
        self.keyword = keyword

    def register_rule(self, rule_key, rule_callback):  # pragma: no cover
        self.cache_key_rules[rule_key] = rule_callback
        return self

    def setup(self, app):  # pragma: no cover
        """Make sure that other installed plugins don't affect the same
        keyword argument and check if metadata is available.
        """
        for other in app.plugins:
            if not isinstance(other, CachePlugin):
                continue
            if other.keyword == self.keyword:
                raise bottle.PluginError(
                    "Found another cache plugin "
                    "with conflicting settings ("
                    "non-unique keyword)."
                )

    def apply(self, callback, context):  # pragma: no cover
        """Implement bottle.py API version 2 `apply` method.
        """
        cache_enabled = getattr(callback, 'cache_info', None)

        if not cache_enabled:
            callback_args = inspect.getfullargspec(context.callback).args

            if self.keyword not in callback_args:
                return callback

            def _wrapped_injected(*args, **kwargs):
                kwargs[self.keyword] = self.backend
                return callback(*args, **kwargs)

            return _wrapped_injected

        if cache_enabled.cache_key_func not in self:
            raise bottle.PluginError(
                'Unregistered cache key function: '
                '{}.'.format(cache_enabled.cache_key_func)
            )

        def wrapper(*args, **kwargs):

            cache_key_fn = self.cache_key_rules[cache_enabled.cache_key_func]
            key = cache_key_fn(bottle.request, context)
            data = self.backend.get(key)

            if data:
                bottle.response.content_type = cache_enabled.content_type
                return ujson.loads(data)

            result = callback(*args, **kwargs)
            if result.status == "200 OK":
                self.backend.set(key, ujson.dumps(result.body), cache_enabled.ttl)
            return result

        return wrapper

    def __repr__(self):  # pragma: no cover
        return "<{} instance at: 0x{:x}>".format(self.__class__, id(self))

    def __str__(self):  # pragma: no cover
        return '<{} instance> {}'.format(
            self.__class__.__name__,
            self.keyword
        )

    def __iter__(self):  # pragma: no cover
        for key in self.cache_key_rules:
            yield key

    def __contains__(self, item):  # pragma: no cover
        return item in self.cache_key_rules

    def __getitem__(self, item):  # pragma: no cover
        return self.cache_key_rules.get(item, None)

    def __setitem__(self, key, value):  # pragma: no cover
        self.cache_key_rules[key] = value

    def __delitem__(self, key):  # pragma: no cover
        del self.cache_key_rules[key]



import bottle
from marshmallow import Schema
from ICECREAM.http import HTTPError
from sqlalchemy_filters import apply_pagination


class PaginateSchema(Schema):
    class Meta:
        fields = ("result", "count", "next", "total_results", "total_page")


class Paginate(object):
    def __new__(cls, query, serializer=None):
        page_number = bottle.request.GET.get('page') or 1
        page_size = bottle.request.GET.get('count') or 10
        query, pagination = apply_pagination(query, page_number=int(page_number),
                                             page_size=int(page_size))
        page_size, page_number, num_pages, total_results = pagination
        _next = cls.is_next_page(page_number, page_size, total_results)
        result = serializer.dump(query)
        __paginate_serializer = PaginateSchema()
        __dict_paginate = {"count": page_number, "next": _next, "total_results": total_results, "total_page": num_pages,
                           "result": result}
        __serialized_result = __paginate_serializer.dump(__dict_paginate)
        return __serialized_result

    @staticmethod
    def is_next_page(page_number, page_size, total_result):
        if total_result - (page_number * page_size) <= 0:
            return False
        return True

    @staticmethod
    def validate_response_count(count):
        if int(count) > 30:
            raise HTTPError(404, "count of page is too big")
        return int(count)



# -*- coding: utf-8 -*-
#
#    Copyright (C) 2015  Papavassiliou Vassilis
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""`bottle_smart_filters` package.
Smart Querystring Params guessing for bottle.py applications.
"""

__version__ = '0.3'

__author__ = 'Papavassiliou Vassilis'
__date__ = '2016-9-14'

__all__ = ('SmartFiltersPlugin',)

import ast
from functools import partial
import bottle
import ujson
from mongosql import MongoQuery, MongoQuerySettingsDict
from sqlalchemy_filters import apply_filters

from ICECREAM.http import HTTPResponse, HTTPError
from ICECREAM.paginator import Paginate


class SmartFiltersPlugin(object):
    """Bottle.py application plugin for query string parameters smart detection.
    Attributes:
        keyword (str): The string keyword for application registry.
        filter_set (instance): A QueryFilterSet instance.
    """
    scope = ('plugin', 'middleware')
    api = 2

    def __init__(self, keyword='smart_filters', multiple_separator=',', json_identifiers=None):
        self.keyword = keyword
        self.separator = multiple_separator
        self.json_identifiers = set(json_identifiers) if json_identifiers else {'[', '{'}

    def setup(self, app):  # pragma: no cover
        """Make sure that other installed plugins don't affect the same
        keyword argument and check if metadata is available.
        """

        for other in app.plugins:
            if not isinstance(other, SmartFiltersPlugin):
                continue
            if other.keyword == self.keyword:
                raise bottle.PluginError("Found another plugin "
                                         "with conflicting settings ("
                                         "non-unique keyword).")

    def apply(self, callback, context):  # pragma: no cover
        """Implement bottle.py API version 2 `apply` method.
        """
        assert context

        def _wrapper(*args, **kwargs):
            """Decorated Injection
            """
            setattr(bottle.request.query, 'smart_filters',
                    lambda: partial(self.filter_set, bottle.request.query)())

            return callback(*args, **kwargs)

        return _wrapper

    def filter_set(self, query_data):
        """Query Filter smart guessing functionality.
        Args:
            query_data (object): A `bottle.FormsDict` instance.
        Returns:
            A dictionary mapping of querystring names -> smart guessed values (if possible or just the values)
        """

        request_data = dict(query_data.iteritems())

        request_filters = {}

        if not request_data:
            return {}

        for alias, value in request_data.items():

            if self.separator not in value or set(value).intersection(self.json_identifiers):
                try:
                    request_filters[alias] = ujson.loads(value)
                except ValueError:
                    request_filters[alias] = value
            else:

                value_list = [val.strip() for val in value.split(self.separator)]
                try:
                    request_filters[alias] = [ujson.loads(val) for val in value_list]

                except ValueError:
                    request_filters[alias] = value_list

        return request_filters


class RestFilter(object):
    def __init__(self, db_session, model, serializer):
        self.db_session = db_session
        self.model = model
        self.serializer = serializer

    def filters(self):
        filter_spec = self.__get_uri_query()
        query = self.db_session.query(self.model)
        try:
            filtered_query = apply_filters(query, filter_spec)
        except Exception as e:
            raise HTTPResponse(status=200, body=e.args)
        result = Paginate(filtered_query, self.serializer)
        return result

    def __get_uri_query(self):
        try:
            filters = bottle.request.query.smart_filters()
            filter_spec = filters.get("filters")
            filter_spec = ast.literal_eval(filter_spec)
        except Exception as e:
            raise HTTPResponse(status=404, body="url_is_not_valid")
        return filter_spec


def get_params_from_url(params: list):
    fields_dict = {}
    for param in params:
        search_field = bottle.request.query.smart_filters().get(param)
        if search_field:
            fields_dict[str(param)] = str(search_field.encode("ISO-8859-1").decode("utf-8"))
        else:
            raise HTTPResponse(status=403, body="not_valid_field")
    return fields_dict


def get_query_from_url(query: str):
    filter_query = bottle.request.query[query]
    query = ast.literal_eval(filter_query.encode("ISO-8859-1").decode("utf-8"))
    return query


class MongoFilter(object):
    def __init__(self, model, query, mongo_filter_query):
        self.query = query
        self.model = model
        self.mongo_filter_query = mongo_filter_query

    def filter(self):
        try:
            result = self.model.mongoquery(self.query).query(**self.mongo_filter_query).end()
            return result
        except Exception as e:
            raise HTTPError(403, e.args)



import logging
import mimetypes
import os
import uuid
import rootpath
from ICECREAM.http import HTTPError
from settings import media_path

logger = logging.getLogger()
rootpath.append()


def upload(data, files_key='files', image=True):
    try:
        files = data.get(files_key)
        for file in files:
            # extension = os.path.splitext(file.filename)[1]
            content_type = file.content_type
            extension = mimetypes.guess_extension(content_type)
            if image:
                check_image_extension(extension)
            unique_file_name = str(uuid.uuid4()) + extension
            file.filename = unique_file_name
            save_path = os.path.join(media_path)
            if not os.path.exists(save_path):
                os.makedirs(save_path)
            file.save(save_path)
            return unique_file_name

    except Exception as error:
        raise error


def remove(file_name):
    try:
        path = os.path.join(media_path)
        os.remove(path + file_name)
    except Exception as e:
        logger.error(e)


def check_image_extension(ext):
    if ext not in ('.png', '.jpg', '.jpeg', '.jpe'):
        raise HTTPError(403, "image_type_forbidden")



from marshmallow import Schema
from marshmallow_sqlalchemy import SQLAlchemyAutoSchema

from ICECREAM.http import HTTPError


def validate_data(serializer: Schema, data: {}):
    validation_errors = serializer.validate(data)
    if validation_errors:
        raise HTTPError(403, validation_errors)
    return True





# icecream framework settings
import os
import rootpath
from dotenv import load_dotenv, find_dotenv

rootpath.append()
load_dotenv(find_dotenv())
project_root = rootpath.detect()
media_path = project_root + os.getenv('media_files')
# you can using instead of os.getenv  media_files = /statics/media/
DEBUG: bool = True

searches_index = [
    # ('users', 'search_vector', ['user_name']),
]

apps = [
    'app_user.urls.USERApp'
]
rules_file = os.getenv("rules_file")
roles_file = os.getenv("roles_file")
default_address = {
    'host': os.getenv('host') or '127.0.0.1',
    'port': os.getenv('port') or 8888,
}
redis_cache = {
    'redis_host': os.getenv('redis_host') or '127.0.0.1',
    'redis_port': (os.getenv('redis_port')) or 6379,
    'redis_pass': os.getenv('redis_pass') or None,
}
database = {
    'db_user': os.getenv('db_user'),
    'db_pass': os.getenv('db_pass'),
    'db_host': os.getenv('db_host'),
    'db_port': os.getenv('db_port'),
    'db_name': os.getenv('db_name'),
    'db_type': os.getenv('db_type') or "sqlite"
}
project_secret = os.getenv('project_secret')
jwt_ttl = os.getenv('jwt_ttl') or 64000
sentry_dsn = os.getenv('sentry_dsn')



import inspect

import bottle
from sqlalchemy.exc import SQLAlchemyError

from ICECREAM.db_initializer import DataBaseConnectionManager

if not hasattr(bottle, 'PluginError'):
    class PluginError(bottle.BottleException):
        pass


    bottle.PluginError = PluginError


class DBInjectorPlugin(object):
    name = 'sqlalchemy'
    api = 2

    def __init__(self, keyword='db_session', db_type='sqlite'):
        self.keyword = keyword
        self.db_type = db_type

    def setup(self, app):
        for other in app.plugins:
            if not isinstance(other, DBInjectorPlugin):
                continue
            if other.keyword == self.keyword:
                raise bottle.PluginError(
                    "Found another SQLAlchemy plugin with " "conflicting settings (non-unique keyword).")
            elif other.name == self.name:
                self.name += '_%s' % self.keyword

    def inspect_parameters(self, callback):
        try:
            inspect.signature(callback)
            # check if inspect.signature exists
        except AttributeError:
            argspec = inspect.getfullargspec(callback)
            parameters = argspec.args
            accept_kwargs = argspec.keywords
        else:
            parameters = inspect.signature(callback).parameters
            accept_kwargs = any(p.kind == inspect.Parameter.VAR_KEYWORD
                                for p in parameters.values())
        return accept_kwargs, parameters

    def apply(self, callback, route):
        if bottle.__version__.startswith('0.9'):
            _callback = route['callback']
        else:
            _callback = route.callback

        accept_kwargs, parameters = self.inspect_parameters(_callback)
        if not (accept_kwargs or self.keyword in parameters):
            return callback

        def wrapper(*args, **kwargs):
            db_manager = DataBaseConnectionManager(db_type=self.db_type)
            session = db_manager.get_db_session()
            try:
                session.expire_on_commit = True
                kwargs['db_session'] = session
                rtn = callback(*args, **kwargs)
                kwargs['db_session'].commit()
                return rtn
            except(SQLAlchemyError, bottle.HTTPError, Exception):
                session.rollback()
                raise
            finally:
                session.close()

        return wrapper


Plugin = DBInjectorPlugin



import json
from alembic import command
from alembic.config import Config
from webtest import TestApp
from .core_manager import Core


class APIResponse(object):
    def __init__(self):
        self.status = None
        self.status_code = None
        self.json = None
        self.body = None
        self.content_type = None


class Client(object):
    def __init__(self):
        self.core = Core(db_type='memory').execute_wsgi()
        self.test_core = TestApp(self.core)

    def migrate_in_memory(self, migrations_path, alembic_ini_path=None, connection=None, revision="head"):
        config = Config(alembic_ini_path)
        config.set_main_option('script_location', migrations_path)
        if connection is not None:
            config.attributes['connection'] = connection
        command.upgrade(config, revision)

    def get_api(self, api_url, *auth):
        response = APIResponse()
        __api_url = str(api_url)
        if auth:
            self.test_core.set_authorization(auth)
        test_core_response = self.test_core.get(__api_url)
        response.json = test_core_response.json
        response.status = test_core_response.status
        response.status_code = test_core_response.status_code
        response.body = test_core_response.body
        response.content_type = test_core_response.content_type

        return response

    def post_api(self, api_url, data, *auth):
        response = APIResponse()
        __api_url = str(api_url)
        if auth:
            self.test_core.set_authorization(auth)
        test_core_response = self.test_core.post_json(__api_url, params=data)
        response.json = json.dumps(test_core_response.json)
        response.status = test_core_response.status
        response.status_code = test_core_response.status_code
        response.body = test_core_response.body
        return response

    def patch_api(self, api_url, data, *auth):
        response = APIResponse()
        __api_url = str(api_url)
        if auth:
            self.test_core.set_authorization(auth)
        test_core_response = self.test_core.patch_json(__api_url, params=data)
        response.json = json.dumps(test_core_response.json)
        response.status = test_core_response.status
        response.status_code = test_core_response.status_code
        response.body = test_core_response.body
        return response

    def put_api(self, api_url, data, *auth):
        response = APIResponse()
        __api_url = str(api_url)
        if auth:
            self.test_core.set_authorization(auth)
        test_core_response = self.test_core.put_json(__api_url, params=data)
        response.json = json.dumps(test_core_response.json)
        response.status = test_core_response.status
        response.status_code = test_core_response.status_code
        response.body = test_core_response.body
        return response

    def delete_api(self, api_url, data, *auth):
        response = APIResponse()
        __api_url = str(api_url)
        if auth:
            self.test_core.set_authorization(auth)
        test_core_response = self.test_core.delete_json(__api_url, params=data)
        response.json = json.dumps(test_core_response.json)
        response.status = test_core_response.status
        response.status_code = test_core_response.status_code
        response.body = test_core_response.body
        return response



from abc import abstractmethod, ABC

from bottle import Bottle


class BaseApp(ABC):
    """every new app must inherit from  BaseApp"""

    @staticmethod
    @abstractmethod
    def call_router(core):
        pass


class PluginApp(ABC):
    @staticmethod
    @abstractmethod
    def install(core: Bottle):
        pass



# -*- coding: utf-8 -*-
"""`bottle_cache.bases` module.

Provides the main package interfaces for caching.
"""

__author__ = 'Papavassiliou Vassilis'
__date__ = '23-1-2016'

import abc
import six
from . import __version__ as version


class CacheError(Exception):
    """Raises when a caching error occurs (i.e connection error)
    """
    pass


@six.add_metaclass(abc.ABCMeta)
class BaseCacheBackend(object):
    """Base class interface.
    """

    api_version = tuple(map(int, version.split('.')))

    __slots__ = ('backend', 'key_tpl')

    def __init__(self, backend_client, key_tpl=None):
        self.backend = backend_client
        self.key_tpl = key_tpl

    @abc.abstractmethod
    def get(self, key):
        """Retrieve from cache based on key.
        Args:
            key(basestring): The cached object key string.

        Returns:
            'Hiredis' parsed python object (dict) if key exists or None
        """
        pass

    @abc.abstractmethod
    def set(self, key, value, ttl=None):
        """Set a value in cache under a specific key, with time to live

        Args:
            key (str): The cache key string.
            value (dict): The dict value.
            ttl (int): The time to live. If none key never expires.

        Returns:
            Cache instance.
        """
        pass

    @abc.abstractmethod
    def remove(self, key):
        """Remove a key/value from cache.

        Args:
            key (str): The cache key.

        Returns:
            Cache instance.
        """

    @abc.abstractmethod
    def clear(self):
        """Removes all key/ values from cache storage.

        Returns:
            Cache instance.
        """

    def key_func(self, key):  # pragma: no cover
        return self.key_tpl.format(key)



try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup
setup(
    name='icecreamy',
    version='0.0.6.2',
    author='xenups',
    include_package_data=True,
    author_email='xenups@gmail.com',
    packages=['ICECREAM', 'ICECREAM', 'ICECREAM.api_cache', 'ICECREAM.migration_tool', 'ICECREAM.models',
              'ICECREAM.statics', 'ICECREAM.assets'],
    url='https://github.com/xenups/ICECREAM',
    license='LICENSE.txt',
    scripts=['make_project.py'],
    description='restful framework with bottle',
    entry_points={
        'console_scripts': ['make_project=ICECREAM.makeproject:make_project'],
    },
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    install_requires=[
        "bottle-jwt2",
        "alembic",
        "appdirs",
        "bottle-jwt2",
        "bottle",
        "codecov",
        "coverage",
        "cryptography",
        "gevent",
        "greenlet",
        "gunicorn",
        "jdatetime",
        "Jinja2",
        "jsonschema",
        "marshmallow",
        "marshmallow-sqlalchemy",
        "mongosql",
        "nose",
        "psycopg2-binary",
        "PyJWT",
        "python-dateutil",
        "python-dotenv",
        "python-throttle",
        "ratelimit",
        "redis",
        "rootpath",
        "sentry-sdk",
        "simplerbac",
        "sqla-wrapper",
        "sqlacodegen",
        "SQLAlchemy==1.3.23",
        "sqlalchemy-filters",
        "SQLAlchemy-Searchable",
        "SQLAlchemy-Utils",
        "tornado",
        "tox",
        "virtualenv",
        "WebTest",
        "Werkzeug",
        "ujson",

    ],
)



from sqlalchemy.orm import Session
from sqlalchemy.orm.exc import NoResultFound
#########need to be refactor################
from ICECREAM.http import HTTPError


def get_nested_list_data(model, data, db_session):
    if not data:
        return []
    _list_object = []
    if hasattr(model(), "id"):
        for _object in data:
            actor_obj = get_object_or_404(model, db_session, model.id == _object.get("id"))
            _list_object.append(actor_obj)
        return _list_object
    HTTPError(400, "Model_should_has_id")


def get_nested_data(model, data, db_session):
    if data is not None:
        source_obj = get_object_or_404(model, db_session, model.id == data["id"])
        return source_obj.id
    return None


def get_or_create(model, session: Session, *args, **kwargs):
    model_object = session.query(model).filter(*args, **kwargs).first()
    if model_object is not None:
        return model_object
    try:
        instance = model(**kwargs)
        session.flush()
    except Exception as msg:
        session.rollback()
        raise (msg)

    return instance


def get_object(model, session, *args, **kwargs):
    """
    Use get() to return an object, return None if object does not exist.
    """
    try:
        model_object = session.query(model).filter(*args, **kwargs).first()
        return model_object
    except Exception as e:
        return None


def get_object_or_404(model, session, *args, **kwargs):
    """
    Use get() to return an object, or raise a Http404 exception if the object
    does not exist.
    """
    model_object = session.query(model).filter(*args, **kwargs).first()
    session.commit()
    if model_object:
        return model_object
    raise HTTPError(404, body=model().__class__.__name__ + "_Not_Found")


def set_objects_limit(list_object: [], limit: int, session: Session):
    """
        Use set_objects_limit to clear last element and hold objects limit count
    """
    offset = (limit - 1) * -1
    if offset == 0:
        [session.delete(_object) for _object in list_object[:]]
    [session.delete(_object) for _object in list_object[:offset]]


def is_object_exist_409(model, session, *args, **kwargs):
    """
        Use is_object_exist_409 if object exist raise Http409.
    """
    model_object = session.query(model).filter(*args, **kwargs).first()
    if model_object:
        raise HTTPError(409, body=model().__class__.__name__ + "_Already_exist")
    return None



import sys
import os

import sqlalchemy_utils
from alembic import context
from sqlalchemy import pool
from logging.config import fileConfig
from sqlalchemy import engine_from_config

config = context.config
sys.path.insert(0, os.path.dirname(os.path.dirname(__file__)))
from ICECREAM.db_initializer import Base, DataBaseConnectionManager


def load_all(module_name):
    import os
    try:
        directories = []
        path = os.getcwd()
        for root, dirs, files in os.walk(path):
            for file in files:
                script = '{}.py'.format(module_name)
                if file.endswith(script):
                    filepath = os.path.join(root, file)
                    if 'env' not in filepath:
                        dir_address = str(os.path.dirname(filepath))
                        directory_name = dir_address.split(str(os.sep))[-1]
                        if directory_name != 'models':
                            directories.append(directory_name)

        for app in directories:
            print(app)
            __import__('{}.{}'.format(app, module_name),
                       fromlist=['*'])
    except Exception as e:
        print(e)


target_metadata = Base.metadata
url = DataBaseConnectionManager().db.url
load_all('models')
config.set_main_option('sqlalchemy.url', url)
fileConfig(config.config_file_name)


def run_migrations_offline():
    url = config.get_main_option("sqlalchemy.url")
    context.configure(
        url=url,
        target_metadata=target_metadata,
        render_item=render_item,
        literal_binds=True,
        dialect_opts={"paramstyle": "named"},
    )

    with context.begin_transaction():
        context.run_migrations()


def render_item(type_, obj, autogen_context):
    """Apply custom rendering for selected items."""
    autogen_context.imports.add("import sqlalchemy_utils")

    return False


def run_migrations_online():
    connectable = engine_from_config(
        config.get_section(config.config_ini_section),
        prefix="sqlalchemy.",
        poolclass=pool.NullPool,
    )

    with connectable.connect() as connection:
        context.configure(
            connection=connection,
            target_metadata=target_metadata,
            render_item=render_item,

        )

        with context.begin_transaction():
            context.run_migrations()


if context.is_offline_mode():
    run_migrations_offline()
else:
    run_migrations_online()



import bottle
from functools import wraps
from time import time
from bottle import request
from sqlalchemy.exc import SQLAlchemyError

from ICECREAM.cache import RedisCache
from ICECREAM.db_initializer import DataBaseConnectionManager
import logging

logger = logging.getLogger()


def timing(f):
    @wraps(f)
    def wrap(*args, **kw):
        ts = time()
        result = f(*args, **kw)
        te = time()
        logger.info('func:%r args:[%r, %r] took: %2.4f sec' % (f.__name__, args, kw, te - ts))
        return result

    return wrap


def pass_data(func):
    def wrapper(*args, **kwargs):
        if request.json is not None:
            kwargs['data'] = request.json
        elif request.forms is not None:
            my_data = {}
            data_list = request.forms.dict
            for key in data_list.keys():
                my_data[key] = data_list[key][0]
            if request.files is not None and request.files.dict is not None:
                my_data['files'] = request.files.dict.get('files')

            kwargs['data'] = my_data

        rtn = func(*args, **kwargs)
        return rtn

    return wrapper


@bottle.route('/<:re:.*>', method='OPTIONS')
def cors(fn):
    def _cors(*args, **kwargs):
        # set CORS headers
        bottle.response.headers['Access-Control-Allow-Origin'] = '*'
        bottle.response.headers['Access-Control-Allow-Methods'] = 'GET, POST, PUT, OPTIONS'
        bottle.response.headers[
            'Access-Control-Allow-Headers'] = 'Origin, Accept, Content-Type, X-Requested-With, X-CSRF-Token'

        if bottle.request.method != 'OPTIONS':
            # actual request; reply with the actual response
            return fn(*args, **kwargs)

    return _cors


def db_handler(func):
    def wrapper(*args, **kwargs):
        session = DataBaseConnectionManager().get_db_session()
        try:
            session.expire_on_commit = False
            kwargs['db_session'] = session
            rtn = func(*args, **kwargs)
            kwargs['db_session'].commit()
            return rtn
        except(SQLAlchemyError, bottle.HTTPError, Exception):
            session.rollback()
            raise

    return wrapper


def debug(fn):
    def wrapper(*args, **kwargs):
        logger.info("Entering {:s}".format(fn.__name__))
        result = fn(*args, **kwargs)
        logger.info("Finished {:s}".format(fn.__name__))
        return result

    return wrapper


def clear_cache(cached_url):
    def decorator(function):
        def wrapper(*args, **kwargs):
            redis = RedisCache()
            redis.clear_ns(ns=cached_url)
            result = function(*args, **kwargs)
            return result
        return wrapper
    return decorator



import os
import csv
import bottle
from csv import reader
from rbac.acl import Registry
from sqlalchemy.orm import Session
from ICECREAM.http import HTTPError
from app_user.models import User
from rbac.proxy import RegistryProxy
from rbac.context import IdentityContext

from settings import rules_file, roles_file

project_root = os.path.abspath(os.curdir)
roles_path = project_root + "/" + roles_file
rules_path = project_root + "/" + rules_file


class ACLHandler(object):
    def __init__(self, Resource):
        self.__acl = RegistryProxy(Registry())
        self.__set_roles()
        self.__acl.add_resource(Resource)
        self.__resource = Resource
        self.__set_rules()

    @staticmethod
    def _read_file(file_name: str) -> list:
        with open(file_name, newline='') as f:
            _reader = csv.reader(f)
            data = list(_reader)
            return data

    def add_resource(self, Resource):
        self.__acl.add_resource(Resource)
        self.__resource = Resource

    def __set_roles(self, ):
        roles = self._read_file(roles_path)[0]
        for role in roles:
            self.__acl.add_role(str(role))

    def __set_rules(self):
        rules = self._read_file(rules_path)
        _resource_name = str(self.__resource().__class__.__name__).lower().strip()
        for rule in rules:
            if str(rule[2]).lower() == _resource_name:
                self.__acl.allow(role=str(rule[0]).strip(), operation=str(rule[1]).strip(), resource=self.__resource)

    def get_identity(self, current_user):
        identity = IdentityContext(self.__acl, lambda: current_user.get_roles())
        return identity


def get_rules_json(user):
    roles = user.get_roles()
    rules = []
    with open(rules_path, 'r') as read_obj:
        csv_reader = reader(read_obj)
        for row in csv_reader:
            # rule = {"role": row[0], "rule": row[1], "object": row[2]}
            if row[0] in roles:
                rules.append(row[1])
    return rules


def get_roles_json():
    rules = []
    with open(roles_path, 'r') as read_obj:
        csv_reader = reader(read_obj)
        for row in csv_reader:
            rule = {"role": row}
            rules.append(rule)
    return rules


def get_roles_list():
    roles = ([a['role'] for a in get_roles_json()])
    return roles[0]


def get_user_identity(db_session, model):
    user = bottle.request.get_user()
    current_user = db_session.query(User).get(user['id'])
    aclh = ACLHandler(Resource=model)
    aclh.add_resource(model)
    identity = aclh.get_identity(current_user)
    return identity


def validate_permission(rule: str, db_session: Session, model):
    identity = get_user_identity(db_session, model)
    if identity.check_permission(rule, model):
        return True
    raise HTTPError(status=403, body="Access_denied")


def validate_permissions(rules: [], db_session: Session, model):
    identity = get_user_identity(db_session, model)
    for rule in rules:
        if not identity.check_permission(rule, model):
            raise HTTPError(status=403, body="Access_denied")
    return True
